# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 15.11.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 11, task 1.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation
# pylint: disable=redefined-outer-name
# pylint: disable=import-error

import os

import numpy as np
import matplotlib.pyplot as plt
import cv2
import skimage.morphology as morph
import seaborn

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None,
               fixed_range=True):
  """Plot image"""
  fig = plt.figure(fig_num)
  if fixed_range:
    vmin = np.iinfo(image.dtype).min
    vmax = np.iinfo(image.dtype).max
  else:
    vmin = int(np.min(image))
    vmax = int(np.max(image))
  plt.imshow(image, cmap=colormap, interpolation='none', vmin=vmin, vmax=vmax)
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num

def main():
  """Segmentation of cells"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 11')
  print('Task 1')
  print('-'*80)

  # Configurations
  result_dir = 'results/images'
  input_file = '../../images/chromosomes.jpg'
  thresh_val = 240
  min_thresh_val = 150

  image = cv2.imread(input_file, cv2.IMREAD_GRAYSCALE)
  fig_num = plot_image(image, 0,
                       write_file=os.path.join(result_dir, 't02_chromosomes.png'))

  fig = plt.figure(fig_num)
  _, _, _ = plt.hist(image.flatten(), 256)
  fig.savefig(os.path.join(result_dir, 't02_histogram.png'),
              bbox_inches='tight', pad_inches=0)
  fig_num += 1

  otsu_val, otsu_thresholded = cv2.threshold(image, 0, 255,
                                             cv2.THRESH_BINARY+cv2.THRESH_OTSU)
  fig_num = plot_image(otsu_thresholded, fig_num,
                       write_file=os.path.join(result_dir, 't02_otsu.png'))
  print(otsu_val)

  man_thresholded = image > thresh_val
  fig_num = plot_image(man_thresholded.astype(np.uint8)*255, fig_num,
                       write_file=os.path.join(result_dir, 't02_manual.png'))

  prev_region_image = man_thresholded
  proposed_regions = image > min_thresh_val
  fig_num = plot_image(proposed_regions.astype(np.uint8)*255, fig_num,
                       write_file=os.path.join(result_dir, 't02_proposed.png'))

  niter = 0
  while True:
    dilated_regions = morph.binary_dilation(prev_region_image, np.ones((3, 3)))
    new_region_image = dilated_regions & proposed_regions

    diff = np.linalg.norm(new_region_image - prev_region_image)
    print(niter)
    print(diff)
    if diff < 1:
      break
    prev_region_image = new_region_image.copy()
    niter += 1

  fig_num = plot_image(new_region_image.astype(np.uint8)*255, fig_num,
                       write_file=os.path.join(result_dir, 't02_segmented.png'))

  plt.show()

if __name__ == "__main__":
  main()
